local L = LibStub("AceLocale-3.0"):GetLocale("Elephant")
local Elephant = LibStub("AceAddon-3.0"):GetAddon("Elephant")

local function Prat_CaptureTime()
	if Elephant.db.profile.forcetimestamp then
		return time()
	else
		return nil
	end
end

--[[
Handles messages sent by the WoW engine
as well as the ones sent by Prat.
]]
local function HandleMessage(prat_msg, event, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9)
	if not Elephant.db.profile.prat and prat_msg then return end
	if not Elephant.db.profile.events[event] then return end
	
	local msg, msg2
	--[[ Channels ]]
	if event == "CHAT_MSG_CHANNEL" or event == "CHAT_MSG_CHANNEL_NOTICE" then
		local larg9 = string.lower(arg9)
		local cIndex
		for k,v in pairs(L['generalchats']) do
			if (larg9 == k) or string.find(larg9, k .. " - ") then
				cIndex = k
				break
			end
		end
		if cIndex == nil then cIndex = larg9 end
		
		if Elephant:IsFiltered(cIndex) then return end
		
		if event == "CHAT_MSG_CHANNEL" then
			if not Elephant.dbpc.char.logs[cIndex].enabled then return end
			
			local msg
			if prat_msg then
				msg = {
					['time'] = Prat_CaptureTime(),
					['prat'] = prat_msg
				}
			else
				msg = {
					['time'] = time(),
					['arg1'] = arg1,
					['arg6'] = arg6,
					['arg9'] = arg9
				}
				if arg2 ~= "" then
					msg['arg2'] = arg2
				end
			end
			Elephant:CaptureNewMessage(msg, cIndex)
		end
		
		if event == "CHAT_MSG_CHANNEL_NOTICE" then
			local larg9 = string.lower(arg9)
			local cIndex
			for k,v in pairs(L['generalchats']) do
				if (larg9 == k) or string.find(larg9, k .. " - ") then
					cIndex = k
					break
				end
			end
			if cIndex == nil then cIndex = larg9 end
			
			if arg1 == "YOU_JOINED" then
				Elephant:InitCustomStructure(cIndex, arg9)
				
				if not Elephant.dbpc.char.logs[cIndex].enabled then return end
				
				Elephant:CaptureNewMessage( { ['type'] = "SYSTEM", ['arg1'] = L['customchat']['join'] } , cIndex)
				if Elephant.dbpc.char.currentlogindex == cIndex then
					Elephant:UpdateCurrentLogButtons()
				end
			end
			if arg1 == "YOU_LEFT" then
				if not Elephant.dbpc.char.logs[cIndex] then return end
				if not Elephant.dbpc.char.logs[cIndex].enabled then return end
				
				Elephant:CaptureNewMessage( { ['type'] = "SYSTEM", ['arg1'] = L['customchat']['leave'] } , cIndex)
				Elephant:CaptureNewMessage( { ['arg1'] = " " } , cIndex)
				if Elephant.dbpc.char.currentlogindex == cIndex then
					Elephant:UpdateCurrentLogButtons()
					Elephant:ForceCurrentLogDeleteButtonStatus(true)
				end
			end
		end
	--[[ Default messages ]]
	else
		if prat_msg then
			msg = {
				['time'] = Prat_CaptureTime(),
				['prat'] = prat_msg,
				['type'] = Elephant.db.profile.events[event].type
			}
		else
			msg = {
				['time'] = time(),
				['type'] = Elephant.db.profile.events[event].type,
				['arg1'] = arg1
			}
			
			if	event == "CHAT_MSG_BATTLEGROUND" or
				event == "CHAT_MSG_BATTLEGROUND_LEADER" or
				event == "CHAT_MSG_WHISPER" or
				event == "CHAT_MSG_WHISPER_INFORM" or
				event == "CHAT_MSG_MONSTER_WHISPER" or
				event == "CHAT_MSG_RAID" or
				event == "CHAT_MSG_RAID_LEADER" or
				event == "CHAT_MSG_RAID_WARNING" or
				event == "CHAT_MSG_PARTY" or
				event == "CHAT_MSG_PARTY_LEADER" or
				event == "CHAT_MSG_SAY" or
				event == "CHAT_MSG_MONSTER_SAY" or
				event == "CHAT_MSG_YELL" or
				event == "CHAT_MSG_MONSTER_YELL" or
				event == "CHAT_MSG_OFFICER" or
				event == "CHAT_MSG_GUILD" or
				event == "CHAT_MSG_EMOTE" or
				event == "CHAT_MSG_MONSTER_EMOTE" or
				event == "CHAT_MSG_BN_WHISPER" or
				event == "CHAT_MSG_BN_WHISPER_INFORM" or
				event == "CHAT_MSG_ACHIEVEMENT" or
				event == "CHAT_MSG_GUILD_ACHIEVEMENT"
			then
				msg.arg2 = arg2
			end
			
			if event == "CHAT_MSG_WHISPER" then
				msg.arg6 = arg6
			end
			
			if event == "PARTY_LOOT_METHOD_CHANGED" then
				local method, masterloot_party, masterloot_raid = GetLootMethod()
				
				if method ~= Elephant.defaultConf.lootmethod then
					Elephant.defaultConf.lootmethod = method
					msg.arg1 = L['lootmethod'][method]
				end
				
				if masterloot_party or masterloot_raid then
					local player
					if UnitInRaid("player") then
						player = GetRaidRosterInfo(masterloot_raid)
					else
						if masterloot_party > 0 then
							player = UnitName("party" .. masterloot_party)
						else
							player = UnitName("player")
						end
					end
					
					if player ~= Elephant.defaultConf.masterlooter then
						Elephant.defaultConf.masterlooter = player
						
						msg2 = {
							['time'] = time(),
							['type'] = Elephant.db.profile.events[event].type,
							['arg1'] = format(L['masterlooterchanged'], player)
						}
					end
				else
					Elephant.defaultConf.masterlooter = nil
				end
			end
		end

		local k
		for k in pairs(Elephant.db.profile.events[event].channels) do
			if Elephant.db.profile.events[event].channels[k] ~= 0 and Elephant.dbpc.char.logs[k].enabled then
				Elephant:CaptureNewMessage(msg, k)
				if msg2 ~= nil then
					Elephant:CaptureNewMessage(msg2, k)
				end
			end
		end
	end
end

function Elephant:RegisterEventsRefresh()
	Elephant:UnregisterAllEvents()
	
	if Prat	and Elephant.db.profile.prat then
		Prat.RegisterChatEvent(Prat, Prat.Events.POST_ADDMESSAGE, function(_, _, _, event, text) HandleMessage(text, event) end)
		return
	else
		if not Prat and Elephant.db.profile.prat then
			Elephant:Print("|cffff0000" .. L['noprat'] .. "|r")
		end
		
		local event
		for event,v in pairs(Elephant.db.profile.events) do
			-- Added event as third parameter to be able to retrieve
			-- it in called method
			Elephant:RegisterEvent(event, HandleMessage, nil)
		end
	end
end